package org.apidesign.apifest08.test;

import junit.framework.TestCase;
import org.apidesign.apifest08.currency.*;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Currency;

/**
 * Finish the Convertor API, and then write bodies of methods inside
 * of this class to match the given tasks. To fullfil your task, use the
 * API define in the <code>org.apidesign.apifest08.currency</code> package.
 * Do not you reflection, or other hacks as your code
 * shall run without any runtime permissions.
 */
public class Task1Test extends TestCase {
    private static CurrencyConvertorFactory currencyConvertorFactoryInstance;

    public Task1Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
        currencyConvertorFactoryInstance = ConvertorsFactory.getCurrencyConvertorFactoryInstance();
    }

    @Override
    protected void tearDown() throws Exception {
        currencyConvertorFactoryInstance = null;
    }

    /**
     * Create convertor that understands two currencies, CZK and
     * USD. Make 1 USD == 17 CZK.
     * <p/>
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting USD to CZK and CZK to USD
     */
    public static Convertor createCZKtoUSD() throws ConvertorNotAvailableException {
        final Currency czk = Currency.getInstance("CZK");
        final Currency usd = Currency.getInstance("USD");
        final BigDecimal constant = BigDecimal.ONE.divide(BigDecimal.valueOf(17), 10, RoundingMode.HALF_EVEN);
        return currencyConvertorFactoryInstance.createConvertor(czk, usd, ConversionProperties.create(constant, RoundingMode.HALF_EVEN));
    }

    /**
     * Create convertor that understands two currencies, CZK and
     * SKK. Make 100 SKK == 80 CZK.
     * <p/>
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting SKK to CZK and CZK to SKK
     */
    public static Convertor createSKKtoCZK() throws ConvertorNotAvailableException {
        final Currency skk = Currency.getInstance("SKK");
        final Currency czk = Currency.getInstance("CZK");
        final BigDecimal constant = new BigDecimal("0.8");
        return currencyConvertorFactoryInstance.createConvertor(skk, czk, ConversionProperties.create(constant, RoundingMode.HALF_EVEN));
    }

    /**
     * Use the convertor from <code>createCZKtoUSD</code> method and do few conversions
     * with it.
     */
    public void testCurrencyCZKUSD() throws Exception {
        Convertor c = createCZKtoUSD();
        // convert $5 to CZK using c:

        final int czkDigitsScale = c.getCurrency1().getDefaultFractionDigits();
        final int usdDigitsScale = c.getCurrency2().getDefaultFractionDigits();

        assertEquals("Result is 85 CZK", BigDecimal.valueOf(85).setScale(czkDigitsScale), c.convertCurrency2ToCurrency1(BigDecimal.valueOf(5)));

        // convert $8 to CZK
        assertEquals("Result is 136 CZK", BigDecimal.valueOf(136).setScale(czkDigitsScale), c.convertCurrency2ToCurrency1(BigDecimal.valueOf(8)));

        // convert 1003CZK to USD
        assertEquals("Result is 59 USD", BigDecimal.valueOf(59).setScale(usdDigitsScale), c.convertCurrency1ToCurrency2(BigDecimal.valueOf(1003)));
    }

    /**
     * Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZK() throws Exception {
        Convertor c = createSKKtoCZK();
        // convert 16CZK using c:

        final int skkDigitsScale = c.getCurrency1().getDefaultFractionDigits();
        final int czkDigitsScale = c.getCurrency2().getDefaultFractionDigits();
        
        assertEquals("Result is 20 SKK", BigDecimal.valueOf(20).setScale(skkDigitsScale), c.convertCurrency2ToCurrency1(BigDecimal.valueOf(16)));

        // convert 500SKK to CZK
        assertEquals("Result is 400 CZK", BigDecimal.valueOf(400).setScale(czkDigitsScale), c.convertCurrency1ToCurrency2(BigDecimal.valueOf(500)));
    }

}

