package org.apidesign.apifest08.currency;

import static org.apidesign.apifest08.currency.Assert.notNull;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Currency;

public final class Convertor {
	
	private final Currency first;
	private final Currency second;
	private final BigDecimal rateValue; // a rate between the first currency and the second currency
	public static final BigDecimal one = new BigDecimal(1);	
	
	public Convertor(BigDecimal rateValue, Currency currencyFirst, Currency currencySecond) {
		notNull(currencyFirst, "currencyFirst");
		notNull(currencySecond, "currencySecond");		
		notNull(rateValue, "rateValue");
		
		this.rateValue = rateValue;
		this.first = currencyFirst;
		this.second = currencySecond;
	}
    	
	/**
	 * Converts an amount value between the two currencies of this converter.
	 *  
	 * @param amount an amount
	 * @param fromCurrency an amount currency
	 * @param toCurrency to a target currency
	 * @return a converted amount value
	 * 
	 * @throws ConversionException if the conversion fails
	 * @throws UnsupportedConversionException if the conversion between a given currencies is not supported.
	 */
	public Amount convert(BigDecimal amount, Currency fromCurrency, Currency toCurrency) throws ConversionException {
		notNull(amount, "amount");
		notNull(fromCurrency, "fromCurrency");
		notNull(toCurrency, "toCurrency");
		
		if((fromCurrency != first && fromCurrency != second) || (toCurrency != first && toCurrency != second)) {
			throw new UnsupportedConversionException(fromCurrency, toCurrency);
		}		

		BigDecimal rateValue = getRateValue(fromCurrency, toCurrency);
		BigDecimal result = rateValue.multiply(amount);			
		return new Amount(result, toCurrency);	
	}
	
	private BigDecimal getRateValue(Currency fromCurrency, Currency toCurrency) {		
		
		BigDecimal retVal;
		
		if(first == fromCurrency) {
			retVal = rateValue;
		} else {	
			//reverse rate	
			retVal = one.divide(rateValue, 10 ,RoundingMode.HALF_UP);
		}
		
		return retVal;
	}
}
